<?php

namespace QUI\ERP\Payments\Stripe;

use QUI;
use QUI\ERP\Accounting\Invoice\Invoice;
use QUI\ERP\ErpEntityInterface;
use QUI\ERP\Exception;
use QUI\ERP\Order\AbstractOrder;
use Stripe\PaymentMethod as StripePaymentMethod;
use Stripe\Exception\ApiErrorException;
use QUI\Interfaces\Users\User as UserInterface;

use function mb_strpos;
use function str_replace;

/**
 * Class Utils
 *
 * Utility class for quiqqer/payment-stripe
 */
class Utils
{
    /**
     * Get order amount in the smallest currency unit
     *
     * @param ErpEntityInterface $Source
     * @return float|int
     *
     * @throws Exception
     * @throws QUI\Exception
     * @throws StripeException
     */
    public static function getCentAmount(QUI\ERP\ErpEntityInterface $Source): float|int
    {
        if ($Source instanceof AbstractOrder) {
            $PriceCalculation = $Source->getPriceCalculation();
            $amount = $PriceCalculation->getSum()->precision(2)->get();
            return $amount * 100; // convert to the smallest currency unit
        }

        if ($Source instanceof Invoice) {
            $invoiceAmount = (float)$Source->getAttribute('toPay');
            $AmountValue = new QUI\ERP\Accounting\CalculationValue($invoiceAmount, $Source->getCurrency(), 2);
            return $AmountValue->get() * 100;
        }

        throw new StripeException('Invalid $Source provided.');
    }

    /**
     * Attaches a Stripe PaymentMethod to a Stripe Customer.
     *
     * If the Customer does not have any previous PaymentMethods, the PaymentMethod is automatically
     * set as the default.
     *
     * @param UserInterface $QuiqqerUser
     * @param string $paymentMethodId
     * @return void
     *
     * @throws StripeException
     * @throws ApiErrorException
     * @throws QUI\Exception
     */
    public static function attachPaymentMethodToStripeCustomer(
        UserInterface $QuiqqerUser,
        string $paymentMethodId
    ): void {
        // If ERPUser -> get QUIQQER User
        if ($QuiqqerUser instanceof QUI\ERP\User) {
            $QuiqqerUser = QUI::getUsers()->get($QuiqqerUser->getUUID());
        }

        $stripeCustomerId = $QuiqqerUser->getAttribute(AbstractBasePayment::USER_ATTR_STRIPE_CUSTOMER_ID);

        if (empty($stripeCustomerId)) {
            throw new StripeException([
                'quiqqer/payment-stripe',
                'exception.Utils.attachPaymentMethodToStripeCustomer.no_stripe_customer_id',
                [
                    'userId' => $QuiqqerUser->getUUID()
                ]
            ]);
        }

        Provider::setupApi();
        $StripePaymentMethod = StripePaymentMethod::retrieve($paymentMethodId);

        try {
            $StripePaymentMethod->attach([
                'customer' => $stripeCustomerId
            ]);
        } catch (ApiErrorException $Exception) {
            if (mb_strpos($Exception->getMessage(), 'has already been attached')) {
                return;
            }

            throw $Exception;
        }
    }

    /**
     * Get a payment description for order (shown at Stripe dashboard)
     *
     * @param AbstractOrder $Order
     * @return string
     */
    public static function getPaymentDescriptionForOrder(AbstractOrder $Order): string
    {
        $defaultDescription = $Order->getCustomer()->getLocale()->get(
            'quiqqer/payment-stripe',
            'PaymentIntent.description.default',
            [
                'orderId' => $Order->getPrefixedId()
            ]
        );

        try {
            $template = Provider::getGeneralSetting('payment_description');

            if (empty($template)) {
                return $defaultDescription;
            }

            $placeholders = ['orderId'];

            foreach ($placeholders as $placeholder) {
                switch ($placeholder) {
                    case 'orderId':
                        $replace = $Order->getPrefixedId();
                        break;

                    default:
                        continue 2;
                }

                $template = str_replace('{' . $placeholder . '}', $replace, $template);
            }

            return $template;
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return $defaultDescription;
        }
    }
}
